/* This file is part of Om.  Copyright (C) 2006 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"
#include "cmdline.h"
#include "ConnectWindow.h"
#include "App.h"
#include "Store.h"
#include "Controller.h"
#include "Configuration.h"
#ifdef HAVE_LASH
	#include "LashController.h"
#endif

using namespace OmGtk;


int
main(int argc, char** argv)
{
	Gnome::Canvas::init();
	Gtk::Main gtk_main(argc, argv);
	
	/* Instantiate all singletons */
	App::instantiate();
	Store::instantiate();
	Controller::instantiate(App::instance().control_interface());

	/* Load settings */
	App::instance().configuration()->load_settings();
	App::instance().configuration()->apply_settings();

	#ifdef HAVE_LASH
	lash_args_t* lash_args = lash_extract_args(&argc, &argv);
	#endif
	
	/* Parse command line options */
	gengetopt_args_info args_info;
	if (cmdline_parser (argc, argv, &args_info) != 0)
		return 1;

	if (args_info.engine_url_given)
		Controller::instance().engine_url(args_info.engine_url_arg);

	if (args_info.client_port_given)
		Controller::instance().client_port(args_info.client_port_arg);
	
	//gtk_main.signal_quit().connect(sigc::ptr_fun(cleanup));		

	#ifdef HAVE_LASH
	LashController* lash_controller = new LashController(lash_args);
	#endif
	
	App::instance().connect_window()->start();
	gtk_main.run();

	return 0;
}

