/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLIENTRECORD_H
#define CLIENTRECORD_H

#include <lo/lo.h>

namespace Om {
	

/** Used to store client addresses.
 *
 * This refers to notification band addresses, ie registered clients.  These
 * addresses are NOT used for sending OK or error responses to OSC commands.
 *
 * (TODO: This needs to be made in to a non-OSC-specific abstract base class).
 */
class ClientRecord
{
public:
	ClientRecord(const string& url)
	: m_url(url),
	  m_addr(lo_address_new_from_url(url.c_str()))
	{}

	~ClientRecord()
	{ lo_address_free(m_addr); }

	const string& url()     const  { return m_url; }
	lo_address    address() const  { return m_addr; }

private:
	// Prevent copies (undefined)
	ClientRecord(const ClientRecord&);
	ClientRecord& operator=(const ClientRecord&);
	
	string     m_url;
	lo_address m_addr;
};


} // namespace Om

#endif // CLIENTRECORD_H
