/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "OSCResponder.h"
#include <unistd.h>
#include <lo/lo.h>
#include <iostream>
#include "Om.h"
#include "OmApp.h"
#include "OSCSender.h"

using std::cerr; using std::endl;

namespace Om {


/* Documentation for the OSC namespace implemented by these methods is in
 * OSCSender.cpp so Doxygen builds the page in the right order
 */

	
void
OSCResponder::respond_ok()
{
	if (m_source != NOBODY) {
		if (lo_send(m_source, "/om/response/ok", "i", m_id) < 0) {
			cerr << "Unable to send response " << m_id << "! ("
				<< lo_address_errstr(m_source) << ")" << endl;
		}
	}
}


void
OSCResponder::respond_error(const string& msg)
{
	if (m_source != NOBODY) {
		if (lo_send(m_source, "/om/response/error", "is",m_id, msg.c_str()) < 0) {
			cerr << "Unable to send response " << m_id << "! ("
				<< lo_address_errstr(m_source) << endl;
		}
	}
}


} // namespace Om
