/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef PREPROCESSOR_H
#define PREPROCESSOR_H

#include <cstdlib>
#include <pthread.h>
#include "types.h"
#include "Semaphore.h"

namespace Om {

class SlowEvent;


// FIXME: awful code duplication here, this can probably be implemented by
// wrapping, or possibly extending a Queue

// FIXME: the API of this is very different from Queue, returning NULL
// and whatnot.  not that it really matters, this is pretty Om specific

// FIXME: this "blocking event" crap needs to go.  I'm almost positive it's
// possible for every event

/** Queue of events that need processing before reaching the audio thread.
 *
 * Implemented as a deque (ringbuffer) in a circular array.  Pushing and
 * popping are threadsafe, as long as a single thread pushes and a single
 * thread pops (ie this data structure is threadsafe, but the push and pop
 * methods themselves are not).
 */
class PreProcessor
{
public:
	PreProcessor(size_t size);
	~PreProcessor();

	void       push(SlowEvent* const ev);
	SlowEvent* pop_before(const samplecount time);

	void signal_blocking_event_finished();

	void start();
	void stop();

private:
	// Prevent copies (undefined)
	PreProcessor(const PreProcessor&);
	PreProcessor& operator=(const PreProcessor&);
	
	// Note that it's crucially important which functions access which of these
	// variables, to maintain threadsafeness.
	
	size_t      m_front;         ///< Front of queue
	size_t      m_back;          ///< Back of entire queue (1 past index of back element)
	size_t      m_prepared_back; ///< Back of prepared section (1 past index of back prepared element)
	const size_t m_size;
	SlowEvent**  m_events;
	
	bool            m_thread_exists;
	bool            m_prepare_thread_exit_flag;
	pthread_t       m_prepare_thread;
	Semaphore       m_semaphore; ///< Counting semaphor for driving prepare thread
	pthread_mutex_t m_blocking_mutex;
	pthread_cond_t  m_blocking_cond;

	static void* prepare_loop(void* q) { return ((PreProcessor*)q)->m_prepare_loop(); }
	void* m_prepare_loop();
};


} // namespace Om

#endif // PREPROCESSOR_H
