/* This file is part of Om.  Copyright (C) 2005 Dave Robillard.
 * 
 * Om is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * Om is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef RESPONDER_H
#define RESPONDER_H

#include <string>
using std::string;

namespace Om {

/** Class to handle responding to clients.
 *
 * This is an abstract base class to fully abstract the details of client
 * communication from the internals of the engine (towards the goal of having
 * the core Om engine a generic shared library, which may respond to a
 * "client" through simple function calls rather than OSC).
 *
 * Note that this class only handles sending responses to clients, (ie OK or an
 * error), not notifications (ie new node, disconnection).
 */
class Responder
{
public:
	virtual ~Responder() {}

	virtual void respond_ok() = 0;
	virtual void respond_error(const string& msg) = 0;
};


} // namespace Om

#endif // RESPONDER_H

